--[[
Name: Stitch-1.0
Revision: $Rev: 29334 $
Author(s): Nymbia (nymbia@gmail.com)
Website: http://www.wowace.com/wiki/Stitch-1.0
Documentation: http://www.wowace.com/wiki/Stitch-1.0
SVN: http://svn.wowace.com/wowace/trunk/Stitch-1.0/Stitch-1.0/
Description: Library for tradeskill information access and queueing.
Dependencies: AceLibrary, AceEvent-2.0
License: LGPL v2.1
Copyright (C) 2006-2007 Nymbia

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
]]
local MAJOR_VERSION = "Stitch-1.0"
local MINOR_VERSION = "$Rev: 29334 $"

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end
if not AceLibrary:HasInstance("AceEvent-2.0") then error(MAJOR_VERSION .. " requires AceEvent-2.0") end
local AceEvent = AceLibrary("AceEvent-2.0")
local ENCHANTING_STRING
local locale = GetLocale()
if locale == "enUS" then
	ENCHANTING_STRING = "Enchanting"
elseif locale == "frFR" then
	ENCHANTING_STRING = "Enchantement"
elseif locale == "deDE" then
	ENCHANTING_STRING = "Verzauberkunst"
elseif locale == "koKR" then
	ENCHANTING_STRING = "마법부여"
elseif locale == "zhCN" then
	ENCHANTING_STRING = "附魔"
elseif locale == "zhTW" then
	ENCHANTING_STRING = "é™„é­”"
end
locale = nil
local Stitch = {}
Stitch.hooks = {}
local rescantrade
local rescancraft
-----------------------------------
-- Stuff for Addons to Implement --
-----------------------------------
function Stitch:EnableDataGathering(addon)
	if not tostring(addon) then error("Usage: EnableDataGathering('addon')") end
	self.datagatheraddons[addon] = true
	self:RegisterEvent("TRADE_SKILL_SHOW")
	self:RegisterEvent("CRAFT_SHOW")
	self:RegisterEvent("TRADE_SKILL_UPDATE")
	self:RegisterEvent("CRAFT_UPDATE")
	self:RegisterEvent("CHAT_MSG_SKILL")
	if not self.data then
		self.data = {}
	end
end
function Stitch:Stitch_AutoRescan()
	if InCombatLockdown() then
		return
	end
	if CraftFrame and self.recentcraft == ENCHANTING_STRING then
		self:ScanCraft()
	end
	if TradeSkillFrame then
		self:ScanTrade()
	end
end
function Stitch:DisableDataGathering(addon)
	if not addon then self.data = nil self.datagatheraddons = {} return end
	if not tostring(addon) then error("Usage: DisableDataGathering(['addon'])") end
	self.datagatheraddons[addon] = false
	for k,v in pairs(self.datagatheraddons) do
		if v then
			return
		end
	end
	self:UnregisterEvent("TRADE_SKILL_SHOW")
	self:UnregisterEvent("CRAFT_SHOW")
	self:UnregisterEvent("TRADE_SKILL_UPDATE")
	self:UnregisterEvent("CRAFT_UPDATE")
	self:UnregisterEvent("CHAT_MSG_SKILL")
	self.data = nil
end
function Stitch:EnableQueue(addon)
	if not tostring(addon) then error("Usage: EnableDataGathering('addon')") end
	self.queueaddons[addon] = true
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", "StopCastCheckUnit")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED", "StopCastCheckUnit")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "StopCastCheckUnit")
	if not self.queue then
		self.queue = {}
	end
	self.queueenabled = true
end
function Stitch:DisableQueue(addon)
	if not addon then self.queue = nil self.queueaddons = {} self.queueenabled = false return end
	if not tostring(addon) then error("Usage: DisableDataGathering(['addon'])") end
	self.queueaddons[addon] = false
	for k,v in pairs(self.queueaddons) do
		if v then
			return
		end
	end
	self:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:UnregisterEvent("UNIT_SPELLCAST_FAILED")
	self:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	self.queueenabled = false
	self.queue = nil
end
function Stitch:GetItemDataByIndex(profession, index)
	if (not tonumber(index)) or not profession then error("Usage: GetItemDataByIndex('profession', index)") end
	if self.data[profession] and self.data[profession][index] then
		self:GetNums(self.data[profession][index].profession, self.data[profession][index].index)
		return self.data[profession][index]
	else
		return nil
	end
end
function Stitch:GetItemDataByName(name)
	if not tostring(name) then error("Usage: GetItemDataByName('name')") end
	for k,v in pairs(self.data) do
		for l,w in pairs(v) do
			if (not tonumber(w)) and w["name"] == name then
				self:GetNums(w.profession, w.index)
				return w
			end
		end
	end
	return nil
end
function Stitch:GetItemDataByPartialName(name)
	if not tostring(name) then error("Usage: GetItemDataByPartialName('name')") end
	local result = {}
	for k,v in pairs(self.data) do
		for l,w in pairs(v) do
			if (not tonumber(w)) and string.match(string.lower(w["name"]), string.lower(name)) then
				self:GetNums(w.profession, w.index)
				table.insert(result,w)
			end
		end
	end
	if table.getn(result) == 0 then
		return nil
	else
		return result
	end
end
function Stitch:GetReagentDataByIndex(profession, skillindex, reagentindex)
	if (not tonumber(skillindex)) or (not tonumber(reagentindex)) or (not self.data[profession]) then error("Usage: GetItemDataByIndex('profession', index)") end
	if self.data[profession][skillindex][reagentindex] then
		if AceLibrary:HasInstance("PeriodicTable-3.0") then
			if AceLibrary("PeriodicTable-3.0"):ItemInSet(self.data[profession][skillindex][reagentindex]["link"], "Tradeskill.Mat.BySource.Vendor") then
				self.data[profession][skillindex][reagentindex]["vendor"] = true
			end
		end
		self.data[profession][skillindex][reagentindex]["num"] = GetItemCount(self:GetIDFromLink(self.data[profession][skillindex][reagentindex]["link"]))
		self.data[profession][skillindex][reagentindex]["numwbank"] = GetItemCount(self:GetIDFromLink(self.data[profession][skillindex][reagentindex]["link"]), true)
		return self.data[profession][skillindex][reagentindex]
	else
		return nil
	end
end
function Stitch:GetReagentDataByName(name)
	if not tostring(name) then error("Usage: GetItemDataByName('name')") end
	local result = {}
	for k,v in pairs(self.data) do
		for l,w in pairs(v) do
			if (not tonumber(w)) then
				for m,x in pairs(w) do
					if tonumber(m) and x["name"] == name then
						self:GetNums(w.profession, w.index)
						table.insert(result,w)
					end
				end
			end
		end
	end
	if table.getn(result) == 0 then
		return nil
	else
		return result
	end
end
function Stitch:GetReagentDataByPartialName(name)
	if not tostring(name) then error("Usage: GetItemDataByName('name')") end
	local result = {}
	for k,v in pairs(self.data) do
		for l,w in pairs(v) do
			if (not tonumber(w)) then
				for m,x in pairs(w) do
					if tonumber(m) and string.match(string.lower(x["name"]), string.lower(name)) then
						self:GetNums(w.profession, w.index)
						table.insert(result,w)
					end
				end
			end
		end
	end
	if table.getn(result) == 0 then
		return nil
	else
		return result
	end
end
function Stitch:GetQueueInfo()
	return self.queue
end
function Stitch:GetQueueItemInfo(index)
	self:GetNums(self.data[self.queue[index]["profession"]][self.queue[index]["index"]].profession, self.data[self.queue[index]["profession"]][self.queue[index]["index"]].index)
	return self.data[self.queue[index]["profession"]][self.queue[index]["index"]]
end
function Stitch:RemoveFromQueue(index)
	table.remove(self.queue, index)
	if table.getn(self.queue) == 0 then
		self:ClearQueue()
	end
end
function Stitch:ClearQueue()
	self.queue = {}
	AceEvent:TriggerEvent("Stitch_Queue_Complete")
end
function Stitch:ProcessQueue()
	local tradeskill = GetTradeSkillLine()
	if self.queue[1] and type(self.queue[1]) == "table" and tradeskill == self.queue[1]["profession"] then
		self.queuecasting = true
		return self.oldDoTrade(self.queue[1]["index"], self.queue[1]["numcasts"])
	else
		self.queue = {}
		AceEvent:TriggerEvent("Stitch_Queue_Complete")
	end
end
------------
-- Events --
------------
function Stitch:TRADE_SKILL_SHOW()
	self.recenttrade = GetTradeSkillLine()
	if self.queue[1] and type(self.queue[1]) == "table" and self.recenttrade ~= self.queue[1]["profession"] then
		self:ClearQueue()
	end
	self:ScanTrade()
end
function Stitch:CRAFT_SHOW()
	self.recentcraft = GetCraftName()
	self:ScanCraft()
end
function Stitch:CHAT_MSG_SKILL()
	rescantrade = true
	rescancraft = true
	self:Stitch_AutoRescan()
end
function Stitch:TRADE_SKILL_UPDATE()
	local num = GetNumTradeSkills()
	local tradeskill = GetTradeSkillLine()
	if tradeskill ~= self.recenttrade or (tonumber(num) and self.recenttrade and self.data[self.recenttrade] and self.data[self.recenttrade]["num"] and num ~= self.data[self.recenttrade]["num"]) or rescantrade then
		self:ScanTrade()
	end
	rescantrade = nil
end
function Stitch:CRAFT_UPDATE()
	local num = GetNumCrafts()
	local tradeskill = GetCraftName()
	if tradeskill ~= self.recentcraft or (tonumber(num) and self.recentcraft and self.data[self.recentcraft] and self.data[self.recentcraft]["num"] and num ~= self.data[self.recentcraft]["num"]) or rescancraft then
		self:ScanCraft()
	end
	rescancraft = nil
end
function Stitch:StopCastCheckUnit(unit)
	if unit == "player" then
		self:StopCast()
	end
end
function Stitch:StopCast()
	if self.queuecasting then
		if event ~= "UNIT_SPELLCAST_SUCCEEDED" then
			AceEvent:TriggerEvent("Stitch_Queue_Continue", table.getn(self.queue))
			self.queuecasting = false
			return
		end
		if not self.queue[1] then
			self.queue = {}
			AceEvent:TriggerEvent("Stitch_Queue_Complete")
		end
		self.queue[1].numcasts = self.queue[1].numcasts - 1
		if self.queue[1].numcasts < 1 then
			self:RemoveFromQueue(1)
			if table.getn(self.queue) > 0 then
				AceEvent:TriggerEvent("Stitch_Queue_Continue", table.getn(self.queue))
			else
				AceEvent:TriggerEvent("Stitch_Queue_Complete")
			end
			self.queuecasting = false
		else
			AceEvent:TriggerEvent("Stitch_Queue_Continue", table.getn(self.queue))
		end
	end
end
--------------------
-- Internal Stuff --
--------------------
function Stitch.hooks:DoTradeSkill(orig, index, times)
	if not index then return end
	if self.queueenabled then
		if index then self:AddToQueue(index, times) end
		self:ProcessQueue()
	else
		return orig(index, times)
	end
end
function Stitch:GetNums(profession, index)
	if not profession or not index or not self.data or not self.data[profession] or not self.data[profession][index] then return end
	self.data[profession][index]["numcraftable"] = 1000
	self.data[profession][index]["numcraftablewbank"] = 1000
	
	for i=1,self.data[profession][index].numreagents do
		if AceLibrary:HasInstance("PeriodicTable-3.0") then
			if AceLibrary("PeriodicTable-3.0"):ItemInSet(self.data[profession][index][i]["link"], "Tradeskill.Mat.BySource.Vendor") then
				self.data[profession][index][i]["vendor"] = true
			end
		end
		self.data[profession][index][i]["num"] = GetItemCount(self:GetIDFromLink(self.data[profession][index][i]["link"]))
		self.data[profession][index][i]["numwbank"] = GetItemCount(self:GetIDFromLink(self.data[profession][index][i]["link"]), true)
		if not self.data[profession][index][i]["vendor"] then
			if self.data[profession][index]["numcraftable"] > math.floor(self.data[profession][index][i]["num"]/self.data[profession][index][i]["needed"]) then
				self.data[profession][index]["numcraftable"] = math.floor(self.data[profession][index][i]["num"]/self.data[profession][index][i]["needed"])
			end
			if self.data[profession][index]["numcraftablewbank"] > math.floor(self.data[profession][index][i]["numwbank"]/self.data[profession][index][i]["needed"]) then
				self.data[profession][index]["numcraftablewbank"] = math.floor(self.data[profession][index][i]["numwbank"]/self.data[profession][index][i]["needed"])
			end
		end
	end
	if self.data[profession][index]["numcraftable"] == 1000 and self.data[profession][index]["numcraftablewbank"] == 1000 then
		for i=1,self.data[profession][index].numreagents do
			if self.data[profession][index]["numcraftable"] > math.floor(self.data[profession][index][i]["num"]/self.data[profession][index][i]["needed"]) then
				self.data[profession][index]["numcraftable"] = math.floor(self.data[profession][index][i]["num"]/self.data[profession][index][i]["needed"])
			end
			if self.data[profession][index]["numcraftablewbank"] > math.floor(self.data[profession][index][i]["numwbank"]/self.data[profession][index][i]["needed"]) then
				self.data[profession][index]["numcraftablewbank"] = math.floor(self.data[profession][index][i]["numwbank"]/self.data[profession][index][i]["needed"])
			end
		end
	end
end
function Stitch:GetIDFromLink(link)
	local id = string.match(link, "item:(%d+)")
	return tonumber(id)
end
function Stitch:AddToQueue(index, times)
	if self.queue[1] and self.queue[1]["profession"] ~= self.recenttrade then
		self:ClearQueue()
	end
	if not times then times = 1 end
	table.insert(self.queue, {
		["profession"] = self.recenttrade,
		["index"] = index,
		["numcasts"] = times,
	})
	AceEvent:TriggerEvent("Stitch_Queue_Add")
end
function Stitch:ScanCraft()
	local prof = GetCraftName()
	if prof~=ENCHANTING_STRING then
		return
	end
	if not self.data[prof] then
		self.data[prof] = {}
	end
	self.data[prof]["num"] = GetNumCrafts()
	local shred = false
	for i=1,self.data[prof]["num"] do
		local skillname, _, skilltype = GetCraftInfo(i)
		if skilltype~="header" and skillname and ((not self.data[prof][i]) or (self.data[prof][i]["name"] ~= skillname) or (self.data[prof][i]["difficulty"] ~= skilltype)) then
			self.data[prof][i] = {}
			self.data[prof][i]["name"] = skillname
			self.data[prof][i]["index"] = i
			self.data[prof][i]["profession"] = prof
			self.data[prof][i]["difficulty"] = skilltype
			self.data[prof][i]["nummade"] = 1
			self.data[prof][i]["link"] = GetCraftItemLink(i)
			self.data[prof][i]["numreagents"] = GetCraftNumReagents(i)
			local v1, _, v2, _, v3, _, v4 = GetCraftSpellFocus(i)
			if type(v4) == "string" then
				self.data[prof][i]["tools"] = v1..", "..v2..", "..v3..", "..v4
			elseif type(v3) == "string" then
				self.data[prof][i]["tools"] = v1..", "..v2..", "..v3
			elseif type(v2) == "string" then
				self.data[prof][i]["tools"] = v1..", "..v2
			elseif type(v1) == "string" then
				self.data[prof][i]["tools"] = v1
			end
			self.data[prof][i]["texture"] = GetCraftIcon(i)
			if not self.data[prof][i]["link"] then
				shred = true
			end
			for j=1,self.data[prof][i]["numreagents"] do
				local rname, rtexture, rcount, _ = GetCraftReagentInfo(i,j)
				self.data[prof][i][j] = {}
				self.data[prof][i][j]["name"] = rname
				self.data[prof][i][j]["link"] = GetCraftReagentItemLink(i,j)
				self.data[prof][i][j]["needed"] = rcount
				self.data[prof][i][j]["texture"] = rtexture
				if not self.data[prof][i][j]["link"] then
					shred = true
				end
			end
		elseif skilltype == "header" then
			self.data[prof][i] = nil
		end
	end
	if shred then
		self.data[prof] = nil
		if not AceEvent:IsEventScheduled("Stitch_AutoRescan") then
			AceEvent:ScheduleEvent("Stitch_AutoRescan", self.Stitch_AutoRescan, 1.5,self)
		end
	end
end
function Stitch:ScanTrade()
	local prof = GetTradeSkillLine()
	
	if prof == "UNKNOWN" then
		self.data[prof] = nil
	end
	if not self.data[prof] then
		self.data[prof] = {}
	end
	self.data[prof]["num"] = GetNumTradeSkills()
	local shred = false
	for i=1,self.data[prof]["num"] do
		local skillname, skilltype = GetTradeSkillInfo(i)
		if skilltype~="header" and skillname and ((not self.data[prof][i]) or (self.data[prof][i]["name"] ~= skillname) or (self.data[prof][i]["difficulty"] ~= skilltype)) then
			self.data[prof][i] = {}
			self.data[prof][i]["name"] = skillname
			self.data[prof][i]["index"] = i
			self.data[prof][i]["profession"] = prof
			self.data[prof][i]["difficulty"] = skilltype
			self.data[prof][i]["nummade"] = GetTradeSkillNumMade(i)
			self.data[prof][i]["link"] = GetTradeSkillItemLink(i)
			self.data[prof][i]["numreagents"] = GetTradeSkillNumReagents(i)
			local v1, _, v2, _, v3, _, v4 = GetTradeSkillTools(i)
			if type(v4) == "string" then
				self.data[prof][i]["tools"] = v1..", "..v2..", "..v3..", "..v4
			elseif type(v3) == "string" then
				self.data[prof][i]["tools"] = v1..", "..v2..", "..v3
			elseif type(v2) == "string" then
				self.data[prof][i]["tools"] = v1..", "..v2
			elseif type(v1) == "string" then
				self.data[prof][i]["tools"] = v1
			end
			self.data[prof][i]["texture"] = GetTradeSkillIcon(i)
			if not self.data[prof][i]["link"] then
				shred = true
			end
			if not tonumber(self.data[prof][i]["numreagents"]) then
				shred = true
			end
			for j=1,self.data[prof][i]["numreagents"] do
				local rname, rtexture, rcount, _ = GetTradeSkillReagentInfo(i,j)
				self.data[prof][i][j] = {}
				self.data[prof][i][j]["name"] = rname
				self.data[prof][i][j]["link"] = GetTradeSkillReagentItemLink(i,j)
				self.data[prof][i][j]["needed"] = rcount
				self.data[prof][i][j]["texture"] = rtexture
				if not self.data[prof][i][j]["link"] then
					shred = true
				end
			end
		elseif skilltype == "header" then
			self.data[prof][i] = nil
		end
	end
	if shred then
		self.data[prof] = nil
		if not AceEvent:IsEventScheduled("Stitch_AutoRescan") then
			AceEvent:ScheduleEvent("Stitch_AutoRescan", self.Stitch_AutoRescan, 1.5,self)
		end
	end
end
----------------------
-- AceLibrary Stuff --
----------------------
local function activate(self, oldLib, oldDeactivate)
	if oldLib then
		self.data = oldLib.data
		self.datagatheraddons = oldLib.datagatheraddons
		self.queueaddons = oldLib.queueaddons
		self.queue = oldLib.queue
		self.queuecasting = oldLib.queuecasting
		self.hooks = oldLib.hooks
		self.queueenabled = oldLib.queueenabled
		self.oldDoTrade = oldLib.oldDoTrade
	else
		self.oldDoTrade = DoTradeSkill
		function DoTradeSkill(index, times)
			if self.hooks.DoTradeSkill then
				return self.hooks.DoTradeSkill(self, self.oldDoTrade, index, times)
			else
				return self.oldDoTrade(index, times)
			end
		end
	end
	if not self.data then
		self.data = {}
	end
	if not self.queueenabled then
		self.queueenabled = false
	end
	if not self.queueaddons then
		self.queueaddons = {}
	end
	if not self.datagatheraddons then
		self.datagatheraddons = {}
	end
	if not self.queue then
		self.queue = {}
	end
	if not self.queuecasting then
		self.queuecasting = false
	end
	if oldDeactivate then
		oldDeactivate(oldLib)
	end
end
local function external(self, major, instance)
	if major == "AceEvent-2.0" then
		AceEvent = instance
		AceEvent:embed(self)
		self:UnregisterAllEvents()
		self:CancelAllScheduledEvents()
	end
end
AceLibrary:Register(Stitch, MAJOR_VERSION, MINOR_VERSION, activate, nil, external)
Stitch = nil



--[[
self.data = {
	professionname = {
		num = total,
		index = {
			["name"] = itemname,
			["index"] = index,
			["profession"] = profession,
			["difficulty"] = "optimal",
			["nummade"] = nummade,
			["link"] = link,
			["numreagents"] = num,
			["tools"] = "tools",
			["texture"] = "texture",
			["numcraftable"] = number,--added
			["numcraftablewbank"] = number,--added
			[reagentindex] = {
				["name"] = name,
				["link"] = link,
				["needed"] = num,
				["texture"] = texture,
				["num"] = number,--added
				["numwbank"] = number--added
			},
		}
	}
}
]]
